import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_wni/flutter_wni.dart';
import 'package:webview_flutter/webview_flutter.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:permission_handler/permission_handler.dart';

class WebPage extends StatefulWidget {
  final String url;

  const WebPage({Key? key, required this.url}) : super(key: key);

  @override
  State<WebPage> createState() => _WebPageState();
}

class _WebPageState extends State<WebPage> {
  late WNInterface _interface;

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () => goBack(context),
      child: Stack(
        children: [
          WebView(
            debuggingEnabled: true,
            initialUrl: widget.url,
            userAgent: WNInterface.getUserAgent(),
            javascriptMode: JavascriptMode.unrestricted,
            javascriptChannels: {
              WNInterface.getJavascriptChannel((WNInterfacePayload payload) async {
                // ignore: avoid_print
                print("payload.command: " + payload.command);

                switch (payload.command) {

                  case "wnGetSafeArea":
                    wnGetSafeArea(payload.data);
                    break;

                  case "wnAllowAccess":
                    wnAllowAccess(payload.data);
                    break;

                  case "wnGetStorage":
                    wnGetStorage(payload.data);
                    break;

                  case "wnSetStorage":
                    wnSetStorage(payload.data);
                    break;

                  default:
                    // ignore: avoid_print
                    print("Unknown Command : ${payload.command}");
                    break;
                }
              })
            },
            onWebViewCreated: (WebViewController controller) async {
              _interface = WNInterface(controller: controller, context: context);
              _interface.controller.clearCache();
            },
          ),
        ],
      ),
    );
  }

  Future<bool> goBack(BuildContext context) async {
    if (await _interface.controller.canGoBack()) {
      _interface.controller.goBack();
      return Future.value(false);
    }

    return Future.value(true);
  }

  // WNInterface Methods
  void wnGetSafeArea(data) async {
    final String callback = data["callback"] as String;
    final EdgeInsets safeArea = MediaQuery.of(context).padding;
    final Map<String, dynamic> result = <String, dynamic>{};
    result["top"] = safeArea.top;
    result["left"] = safeArea.left;
    result["right"] = safeArea.right;
    result["bottom"] = safeArea.bottom;

    _interface.call(callback, jsonEncode(result));
  }

  void wnAllowAccess(data) async {
    final String type = data["type"] as String;
    final String callback = data["callback"] as String;
    
    PermissionStatus? permissionStatus;
    
    if (type == "camera") {
      permissionStatus = await Permission.camera.request();
    }
    else if (type == "location") {
      permissionStatus = await Permission.location.request();
    }
    else if (type == "microphone") {
      permissionStatus = await Permission.microphone.request();
    }
    else if (type == "calendar") {
      permissionStatus = await Permission.calendar.request();
    }
    else if (type == "phone") {
      permissionStatus = await Permission.phone.request();
    }
    else if (type == "contacts") {
      permissionStatus = await Permission.contacts.request();
    }
    else if (type == "photos") {
      permissionStatus = await Permission.photos.request();
    }
    else if (type == "notification") {
      permissionStatus = await Permission.notification.request();
    }

    String status = "UNKNOWN";
    if (permissionStatus != null) {
      switch (permissionStatus) {
        case PermissionStatus.denied:
        case PermissionStatus.permanentlyDenied:
          status = "DENIED";
          break;

        case PermissionStatus.granted:
          status = "GRANTED";
          break;

        case PermissionStatus.limited:
          status = "LIMITED";
          break;

        case PermissionStatus.restricted:
          status = "RESTRICTED";
          break;

        default:
          break;
      }
    }

    // ignore: avoid_print
    print("status: " + status);

    _interface.call( callback, jsonEncode({
      "status": status,
    }));
  }

  void wnGetStorage(data) async {
    final String callback = data["callback"] as String;
    final String key = data["key"] as String;
    final prefs = await SharedPreferences.getInstance();
    final String? value = prefs.getString(key);
    
    _interface.call(callback, jsonEncode({
      "status": "SUCCESS",
      "key": key,
      "value": value ?? ""
    }));
  }

  void wnSetStorage(data) async {
    final String callback = data["callback"] as String;
    final String key = data["key"] as String;
    final String value = data["value"] as String;
    final prefs = await SharedPreferences.getInstance();
    prefs.setString(key, value);

    _interface.call(callback, jsonEncode({
      "status": "SUCCESS",
      "key": key,
      "value": value
    }));
  }
}
